@@ -29,7 +29,7 @@
 extern "C" {
 #endif /* __cplusplus */
 
-void gok_keyslotter_on (gboolean bOnOff, int keytype);
+gboolean gok_keyslotter_on (gboolean bOnOff, int keytype);
 void gok_keyslotter_change_number_predictions (int Number, int keytype);
 gboolean gok_keyslotter_add_prediction_keys (GokKeyboard* pKeyboard, int keytype);
 
