@@ -28,7 +28,6 @@
 #include "gok-page-feedbacks.h"
 #include "gok-page-actions.h"
 #include "gok-data.h"
-#include "support.h"
 #include "gok-settings-dialog.h"
 #include "gok-log.h"
 #include "gok-action.h"
@@ -66,14 +65,14 @@ static GokFilledCombo* m_pGokFilledComboFirst;
 *
 * returns: TRUE if the page was properly initialized, FALSE if not.
 **/
-gboolean gok_page_accessmethod_initialize (GtkWidget* pWindowSettings)
+gboolean gok_page_accessmethod_initialize (GladeXML* xml)
 {
 	GokAccessMethod* pAccessMethod;
 	GtkWidget* pComboBox;
 	GtkWidget* pEntryAccessMethodName;
 	GList* items = NULL;
 		
-	g_assert (pWindowSettings != NULL);
+	g_assert (xml != NULL);
 	
 	/* initialize this data */
 	strcpy (m_NameAccessMethod, gok_data_get_name_accessmethod());
@@ -88,7 +87,7 @@ gboolean gok_page_accessmethod_initialize (GtkWidget* pWindowSettings)
 	pAccessMethod = gok_scanner_get_first_access_method();
 	g_assert (pAccessMethod != NULL);
 	
-	pComboBox = lookup_widget (gok_settingsdialog_get_window(), "comboAccessMethods");
+	pComboBox = glade_xml_get_widget (xml,  "comboAccessMethods");
 	g_assert (pComboBox != NULL);
 	
 	while (pAccessMethod != NULL)
@@ -102,20 +101,17 @@ gboolean gok_page_accessmethod_initialize (GtkWidget* pWindowSettings)
 
 	m_bUsChangingName = TRUE;
 	gtk_combo_set_popdown_strings (GTK_COMBO (pComboBox), items);
-	m_bUsChangingName = FALSE;
+	g_list_free (items);
 
 	/* display the current access method */
-	pEntryAccessMethodName = lookup_widget (pWindowSettings, "entryAccessMethodName");
+	pEntryAccessMethodName = glade_xml_get_widget (xml, "entryAccessMethodName");
 	g_assert (pEntryAccessMethodName != NULL);
 	gtk_entry_set_text (GTK_ENTRY (pEntryAccessMethodName), gok_page_accessmethod_get_displayname (m_NameAccessMethod));
+	m_bUsChangingName = FALSE;
+
+	/* update the accessmethod controls */
+	gok_page_accessmethod_method_changed (GTK_EDITABLE (pEntryAccessMethodName));
 
-	/* force the accessmethod widgets to draw only if the current access method is the first one in the list */
-	if (g_ascii_strcasecmp(m_NameAccessMethod, gok_page_accessmethod_get_name (g_list_nth_data (items, 0))) == 0)
-	{
-		gok_page_accessmethod_method_changed (GTK_EDITABLE (pEntryAccessMethodName));
-	}
-	g_list_free (items);
-	
 	return TRUE;
 }
 
@@ -897,7 +893,7 @@ void gok_page_accessmethod_change_controls (gchar* pNameAccessMethod)
 
 	g_assert (pNameAccessMethod != NULL);
 
-	pFrame = lookup_widget (gok_settingsdialog_get_window(), "vboxMain");
+	pFrame = glade_xml_get_widget (gok_settingsdialog_get_xml(), "AccessMethodsTab");
 	g_assert (pFrame != NULL);
 	ControlParent.pWidget = pFrame;
 	ControlParent.Type = CONTROL_TYPE_FRAME;
@@ -922,13 +918,13 @@ void gok_page_accessmethod_change_controls (gchar* pNameAccessMethod)
 			
 			/* display the name in the combo box */
 			m_bUsChangingName = TRUE;
-			pEntryAccessMethodName = lookup_widget (gok_settingsdialog_get_window(), "entryAccessMethodName");
+			pEntryAccessMethodName = glade_xml_get_widget (gok_settingsdialog_get_xml(), "entryAccessMethodName");
 			g_assert (pEntryAccessMethodName != NULL);
 			gtk_entry_set_text (GTK_ENTRY (pEntryAccessMethodName), gok_page_accessmethod_get_displayname (pNameAccessMethod));
 			m_bUsChangingName = FALSE;
 			
 			/* change the access method description text */
-			pLabelDescription = lookup_widget (gok_settingsdialog_get_window(), "labelAmDescription");
+			pLabelDescription = glade_xml_get_widget (gok_settingsdialog_get_xml(), "labelAmDescription");
 			g_assert (pLabelDescription != NULL);
 			gtk_label_set_text (GTK_LABEL (pLabelDescription), pAccessMethod->Description);
 				
