@@ -378,6 +378,7 @@ gboolean gok_scanner_initialize (const gchar *directory, const gchar *accessmeth
 			g_free (complete_path);
 		}
 	}
+	closedir (pDirectoryAccessMethods);
 
 	/* get the rates for all the access methods */
 	gok_scanner_update_rates ();
@@ -514,6 +515,7 @@ gboolean gok_scanner_read_access_method (gchar* Filename)
 		if (pAccessMethod == NULL)
 		{
 			xmlFreeDoc (pDoc);
+			xmlFree (pStringAttributeValue);
 			return FALSE;
 		}
 	}
@@ -522,8 +524,10 @@ gboolean gok_scanner_read_access_method (gchar* Filename)
 		/* access method must have a name attribute*/
 		gok_log_x ("Error: gok_scanner_create_access_method failed: can't find 'name' attribute.\n");
 		xmlFreeDoc (pDoc);
+		xmlFree (pStringAttributeValue);
 		return FALSE;
 	}
+	xmlFree (pStringAttributeValue);
 
 	/* get the display name of the access method */
 	pStringAttributeValue = xmlGetProp (pNodeAccessmethod, (const xmlChar *) "displayname");
@@ -1076,6 +1080,7 @@ void gok_scanner_read_ui_loop (GokControl* pControl, xmlNode* pNode)
 			{
 				gok_log_x ("no type for control!");
 			}
+			xmlFree (pStringAttributeValue);
 
 			pStringAttributeValue = xmlGetProp (pNode, (const xmlChar *) "name");
 			if (pStringAttributeValue != NULL)
@@ -1083,24 +1088,28 @@ void gok_scanner_read_ui_loop (GokControl* pControl, xmlNode* pNode)
 				pControlNew->Name = (gchar*)g_malloc (strlen ((gchar *)pStringAttributeValue) + 1);
 				strcpy (pControlNew->Name, (gchar *)pStringAttributeValue);
 			}
+			xmlFree (pStringAttributeValue);
 			
 			pStringAttributeValue = xmlGetProp (pNode, (const xmlChar *) "size");
 			if (pStringAttributeValue != NULL)
 			{
 				pControlNew->Size = atoi ((gchar *)pStringAttributeValue);
 			}
+			xmlFree (pStringAttributeValue);
 
 			pStringAttributeValue = xmlGetProp (pNode, (const xmlChar *) "border");
 			if (pStringAttributeValue != NULL)
 			{
 				pControlNew->Border = atoi ((gchar *)pStringAttributeValue);
 			}
+			xmlFree (pStringAttributeValue);
 
 			pStringAttributeValue = xmlGetProp (pNode, (const xmlChar *) "spacing");
 			if (pStringAttributeValue != NULL)
 			{
 				pControlNew->Spacing = atoi ((gchar *)pStringAttributeValue);
 			}
+			xmlFree (pStringAttributeValue);
 			
 			pStringAttributeValue = xmlGetProp (pNode, (const xmlChar *) "string");
 			if (pStringAttributeValue != NULL)
@@ -1108,6 +1117,7 @@ void gok_scanner_read_ui_loop (GokControl* pControl, xmlNode* pNode)
 				pControlNew->String = (gchar*)g_malloc (strlen ((gchar *)pStringAttributeValue) + 1);
 				strcpy (pControlNew->String, (gchar *)pStringAttributeValue);
 			}
+			xmlFree (pStringAttributeValue);
 			
 			pStringAttributeValue = xmlGetProp (pNode, (const xmlChar *) "fillwith");
 			if (pStringAttributeValue != NULL)
@@ -1133,54 +1143,63 @@ void gok_scanner_read_ui_loop (GokControl* pControl, xmlNode* pNode)
 					gok_log_x ("Invalid 'fillwith' value: %s!", pStringAttributeValue);
 				}
 			}
+			xmlFree (pStringAttributeValue);
 			
 			pStringAttributeValue = xmlGetProp (pNode, (const xmlChar *) "qualifier");
 			if (pStringAttributeValue != NULL)
 			{
 				pControlNew->Qualifier = gok_scanner_make_type_from_string ((gchar *)pStringAttributeValue);
 			}
+                        xmlFree (pStringAttributeValue);
 			
 			pStringAttributeValue = xmlGetProp (pNode, (const xmlChar *) "groupstart");
 			if (pStringAttributeValue != NULL)
 			{
 				pControlNew->bGroupStart = (xmlStrcmp (pStringAttributeValue, (const xmlChar *)"yes") == 0) ? TRUE : FALSE;
 			}
+                        xmlFree (pStringAttributeValue);
 			
 			pStringAttributeValue = xmlGetProp (pNode, (const xmlChar *) "value");
 			if (pStringAttributeValue != NULL)
 			{
 				pControlNew->Value = atoi ((gchar *)pStringAttributeValue);
 			}
+                        xmlFree (pStringAttributeValue);
 
 			pStringAttributeValue = xmlGetProp (pNode, (const xmlChar *) "min");
 			if (pStringAttributeValue != NULL)
 			{
 				pControlNew->Min = atoi ((gchar *)pStringAttributeValue);
 			}
+                        xmlFree (pStringAttributeValue);
 
 			pStringAttributeValue = xmlGetProp (pNode, (const xmlChar *) "max");
 			if (pStringAttributeValue != NULL)
 			{
 				pControlNew->Max = atoi ((gchar *)pStringAttributeValue);
 			}
+                        xmlFree (pStringAttributeValue);
 
 			pStringAttributeValue = xmlGetProp (pNode, (const xmlChar *) "stepincrement");
 			if (pStringAttributeValue != NULL)
 			{
 				pControlNew->StepIncrement = atoi ((gchar *)pStringAttributeValue);
 			}
+                        xmlFree (pStringAttributeValue);
 
 			pStringAttributeValue = xmlGetProp (pNode, (const xmlChar *) "pageincrement");
 			if (pStringAttributeValue != NULL)
 			{
 				pControlNew->PageIncrement = atoi ((gchar *)pStringAttributeValue);
 			}
+                        xmlFree (pStringAttributeValue);
 
 			pStringAttributeValue = xmlGetProp (pNode, (const xmlChar *) "pagesize");
 			if (pStringAttributeValue != NULL)
 			{
 				pControlNew->PageSize = atoi ((gchar *)pStringAttributeValue);
 			}
+                        xmlFree (pStringAttributeValue);
 
 			pStringAttributeValue = xmlGetProp (pNode, (const xmlChar *) "associated");
 			if (pStringAttributeValue != NULL)
@@ -1188,18 +1207,21 @@ void gok_scanner_read_ui_loop (GokControl* pControl, xmlNode* pNode)
 				pControlNew->NameAssociatedControl = (gchar*)g_malloc (strlen ((gchar *)pStringAttributeValue) + 1);
 				strcpy (pControlNew->NameAssociatedControl, (gchar *)pStringAttributeValue);
 			}
+                        xmlFree (pStringAttributeValue);
 
 			pStringAttributeValue = xmlGetProp (pNode, (const xmlChar *) "associatedstate");
 			if (pStringAttributeValue != NULL)
 			{
 				pControlNew->bAssociatedStateActive = atoi ((gchar *)pStringAttributeValue);
 			}
+                        xmlFree (pStringAttributeValue);
 
 			pStringAttributeValue = xmlGetProp (pNode, (const xmlChar *) "handler");
 			if (pStringAttributeValue != NULL)
 			{
 				pControlNew->Handler = gok_control_get_handler_type ((gchar *)pStringAttributeValue);
 			}
+                        xmlFree (pStringAttributeValue);
 
 			/* hook the new control into the control given */
 			if (pControl->pControlChild == NULL)
@@ -1345,20 +1367,25 @@ GokScannerEffect* gok_scanner_read_effects (xmlNode* pNodeGiven, GokAccessMethod
 				{
 					if (xmlStrcmp (pStringAttributeValue, (const xmlChar *)ArrayCallNames[count]) == 0)
 					{
+						xmlFree (pStringAttributeValue);
 						/* get the attributes for this effect */
 						/* Paramater 1 */
 						pStringAttributeValue = xmlGetProp (pNodeEffect, (const xmlChar *) "param1");
 						if (pStringAttributeValue != NULL)
 						{
-							Param1 = (char *)pStringAttributeValue;
+							Param1 = (gchar*)g_malloc (strlen ((gchar *)pStringAttributeValue) + 1);
+							strcpy (Param1,(gchar *)pStringAttributeValue);
 						}
+						xmlFree (pStringAttributeValue);
 
 						/* Paramater 2 */
 						pStringAttributeValue = xmlGetProp (pNodeEffect, (const xmlChar *) "param2");
 						if (pStringAttributeValue != NULL)
 						{
-							Param2 = (char *)pStringAttributeValue;
+							Param2 = (gchar*)g_malloc (strlen ((gchar *)pStringAttributeValue) + 1);
+							strcpy (Param2,(gchar *)pStringAttributeValue);
 						}
+						xmlFree (pStringAttributeValue);
 
 						/* call name */
 						pName = (char *)xmlGetProp (pNodeEffect, (const xmlChar *) "name");
@@ -1408,6 +1435,7 @@ if (gok_data_get_setting (pAccessMethod->Name, pStringAttributeValue, NULL, &pNa
 										gok_log_x ("Warning: Compare type invalid (%s) in gok_scanner_read_effects!\n", pStringAttributeValue);
 									}
 								}
+								xmlFree (pStringAttributeValue);
 
 								/* compare value */
 								pStringAttributeValue = xmlGetProp (pNodeCompare, (const xmlChar *) "value");
@@ -1467,6 +1495,8 @@ if (gok_data_get_setting (pAccessMethod->Name, pStringAttributeValue, NULL, &pNa
 															pEffectTrue,
 															pEffectFalse,
 															pName);
+						g_free (pName);
+
 						/* add the effect to our list of effects */
 						if (pEffectFirst == NULL)
 						{
