@@ -151,6 +151,11 @@ static void gok_main_display_geometry_error (void);
 static void gok_main_object_state_listener (Accessible* pAccessible);
 static gboolean gok_main_has_xkb_extension ();
 static gboolean gok_main_check_sticky_keys (GtkWidget *widget);
+void gok_main_mouse_button_listener (gint button, gint state, long mods, long timestamp);
+
+/* extern funcs */
+void       gok_input_detach_corepointer (void);
+void       gok_input_restore_corepointer (void);
 
 static gint _screen_height, _screen_width;
 
@@ -314,7 +319,9 @@ static int gok_sig_handler(int sig)
 	default:
 		break;
 	}
-	
+
+	gok_input_restore_corepointer ();
+
 	_exit(1);
 	return -1;  /* this line gets rid of compiler warning */ 
 }
@@ -476,6 +483,26 @@ comparebasenames (gconstpointer a, gconstpointer b)
 	return returncode;
 }
 
+void 
+gok_main_center_corepointer (GtkWidget *window)
+{
+    Display *display;
+    GdkWindow *root;
+    GdkScreen *screen;
+    gint screen_width, screen_height;
+
+    window = gok_main_get_main_window ();
+    display = GDK_WINDOW_XDISPLAY (window->window);
+    screen = gdk_drawable_get_screen (window->window);
+    root = gdk_screen_get_root_window (screen);
+    screen_width = gdk_screen_get_width (screen);
+    screen_height = gdk_screen_get_height (screen);
+
+    XWarpPointer (display, None, 
+		  GDK_WINDOW_XWINDOW (root),
+		  0, 0, 0, 0, screen_width/2, screen_height/2);
+}
+
 /**
 * gok_main_open
 * @argc: ignored
@@ -719,7 +746,7 @@ gok_main_open(gint argc, gchar *argv[])
 	/* initialize the spy routines */
 	gok_spy_open();
 	gok_spy_register_windowchangelistener ((void *)gok_main_window_change_listener);
-	gok_spy_register_mousebuttonlistener ((void *)gok_main_mousebutton_listener);	
+	gok_spy_register_mousebuttonlistener ((void *)gok_main_mouse_button_listener);	
 	gok_spy_register_objectstatelistener ((void *)gok_main_object_state_listener);
 	
 	/* initialize sound */
@@ -770,17 +797,8 @@ gok_main_open(gint argc, gchar *argv[])
 	}
 
 	if (!gok_args.is_login) {
-	        gchar *message = g_strconcat (_("You are using GOK in \'core pointer\' mode."), 
-					      _("The device you are using to operate GOK"
-						"\nis also controlling the system pointer (or \'mouse pointer\')."
-						"\nConflicts with applications\' use of the pointer may interfere"
-						"\nwith your ability to use applications or GOK.\n"
-						"\nWe strongly recommend configuring your input device as an"
-						"\n\'Extended\' input device instead; see GOK Help for more information."),
-					      NULL);
 		gok_main_check_accessibility();
-		gok_main_warn (message, FALSE, FALSE);
-		g_free (message);
+		gok_main_warn_corepointer (FALSE, FALSE, TRUE);
 	}
 	
 	if (!gok_main_check_sticky_keys (m_pWindowMain)) {
@@ -1144,7 +1162,11 @@ void gok_main_button_listener (gint button, gint state, long mods, long timestam
 		SPI_generateMouseEvent (x, y, ename);
 	}
 
-	if (state == 0)
+	if (gok_input_corepointer_detached ())
+	{
+		gok_main_mouse_button_listener (button, state, mods, timestamp);
+	}
+	else if (state == 0)
 	{
 		gok_main_raise_window ();
 
@@ -1221,7 +1243,8 @@ gok_main_raise_window (void)
 *
 * This handler is called each time there is a mouse button event.
 */
-void gok_main_mousebutton_listener (gint button, gint state, long mods, long timestamp)
+void 
+gok_main_mouse_button_listener (gint button, gint state, long mods, long timestamp)
 {
 	gok_log_enter();
 	gok_log("mouse button[%d] state[%d] mods[%lx] timestamp[%ld]",button,state,mods,timestamp);
@@ -1286,7 +1309,7 @@ void gok_main_mousebutton_listener (gint button, gint state, long mods, long tim
 		}
 	}
 
-	gok_log_leave();	
+	gok_log_leave();
 }
 
 /**
@@ -1557,6 +1580,9 @@ gboolean gok_main_ds (GokKeyboard* pKeyboard)
 	/* stop any key flashing */
 	gok_feedback_timer_stop_key_flash();
 	
+	/* we have branched, so we should make sure 'repeat' is off */
+	gok_repeat_disarm ();
+
 	/* stop the scanning process */
 	gok_scanner_stop();
 
@@ -1624,6 +1650,7 @@ GtkWidget* gok_main_create_window (gboolean is_dock)
 {
 	GtkWidget *window1;
 	GtkWidget *fixed1;
+	GtkWidget *event1;
 
 	gok_log_enter();
 					
@@ -2472,6 +2499,9 @@ gok_main_close()
 	GokKeyboard* pKeyboard;
 	GokKeyboard* pKeyboardTemp;
 
+	/* restore the corepointer, if it was changed... */
+	gok_input_restore_corepointer ();
+
 	/* unhook listeners/callbacks */
 	gok_spy_deregister_windowchangelistener ((void *)gok_main_window_change_listener);
 	gok_spy_deregister_objectstatelistener ((void *)gok_main_object_state_listener);
@@ -2825,12 +2855,12 @@ gok_main_check_sticky_keys (GtkWidget *widget)
 	return TRUE;
 }
 
-
-
 static GtkWidget* acd = NULL; /* accessibility check dialog */
 
 static GtkWidget* _corepointer_warning = NULL; /* core pointer warning dialog */
 
+static gboolean _corepointer_warning_posted = FALSE; /* core pointer warning dialog has been posted */
+
 void check_accessibility_cb ( GObject* o, gpointer* data )
 {
 	GnomeClient *client;
@@ -2866,20 +2896,69 @@ gok_main_help_cb (gpointer data)
 	/* TODO: detect error launching help, and give informative message */
 	gnome_help_display_desktop (NULL, "gok", helpuri, NULL, &error);
 }
- 
+
 void
-gok_main_warn (gchar *message, gboolean always, GCallback cancelfunc)
+gok_main_close_warning ()
 {
-	if ((always && !_corepointer_warning) || gok_scanner_current_state_uses_corepointer ()) 
+        gtk_widget_destroy (_corepointer_warning);
+	_corepointer_warning = NULL;
+} 
+
+void
+gok_main_warn_corepointer (gboolean always, gboolean configuring, gboolean opt_out)
+{
+	if (_corepointer_warning_posted == FALSE)
+	{
+		    gchar *core_message1 = _("The device you are using to operate GOK "
+					     "is also controlling the system pointer (or \'mouse pointer\').  "
+					     "Conflicts with applications\' use of the pointer may interfere "
+					     "with your ability to use applications or GOK.\n\n"
+					     "We strongly recommend configuring your input device as an "
+					     "'Extended\' input device instead; see GOK Help for more information.");
+		    gchar *core_message2 = _("The device you are using to control GOK is also controlling the system pointer.\n\n"
+					     "To avoid conflict with applications\' use of the pointer, "
+					     "GOK will now disconnect the device you are using from the "
+					     "system pointer for the duration of your GOK session.\n\n"
+					     "Alternatively, you may configure GOK to use another "
+					     "input device instead; see Help for details.");
+		    /* no longer used! */
+		    gchar *core_message3 = g_strconcat (_("You appear to be configuring GOK to use \'core pointer\' mode."), 
+							core_message1,
+							NULL);
+		    gchar *core_message4 = g_strconcat (_("You appear to be configuring GOK to use \'core pointer\' mode."), 
+							core_message2, NULL);
+
+		    if (gok_input_ext_devices_exist ())
+		    {
+			_corepointer_warning_posted = gok_main_warn (core_message2, always, gok_input_restore_corepointer, gok_input_detach_corepointer, opt_out);
+		    }
+		    else
+		    {
+			    if (configuring)
+				    _corepointer_warning_posted = gok_main_warn (core_message1, always, gok_main_close_warning, NULL, FALSE);
+			    else 
+				    _corepointer_warning_posted = gok_main_warn (core_message3, always, gok_main_close_warning, NULL, FALSE);
+		    }		    
+		    g_free (core_message3);
+		    g_free (core_message4);
+	}
+}
+
+gboolean
+gok_main_warn (gchar *message, gboolean always, GCallback cancelfunc, GCallback okfunc, gboolean opt_out)
+{
+	if (always || gok_scanner_current_state_uses_corepointer ()) 
 	{
 		GtkWidget*	button;
 		gint         response_id;
 		gboolean 	returnCode = TRUE;
-		
+
+		if (opt_out && okfunc) (*okfunc) ();
+
 		if (_corepointer_warning == NULL)
 		{
 			_corepointer_warning = gtk_message_dialog_new (
-				NULL, 
+  				NULL, 
 				GTK_DIALOG_DESTROY_WITH_PARENT,
 				GTK_MESSAGE_WARNING, 
 				GTK_BUTTONS_NONE,			
@@ -2899,7 +2978,7 @@ gok_main_warn (gchar *message, gboolean always, GCallback cancelfunc)
 			
 			if (cancelfunc != NULL)
 			{
-			    button = gtk_button_new_from_stock(GTK_STOCK_OK);
+			    button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
 			    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(_corepointer_warning)->action_area),
 					       button);
 			    g_signal_connect_swapped (G_OBJECT (button), "clicked", 
@@ -2909,14 +2988,17 @@ gok_main_warn (gchar *message, gboolean always, GCallback cancelfunc)
 			gtk_container_add (GTK_CONTAINER (GTK_DIALOG(_corepointer_warning)->action_area),
 					   button);
 			g_signal_connect_swapped (G_OBJECT (button), "clicked", 
-						  G_CALLBACK (gtk_widget_destroy), _corepointer_warning);
+						  (okfunc == NULL || opt_out) ? G_CALLBACK (gok_main_close_warning) : okfunc, 
+						  _corepointer_warning);
 			
 			/* reset hint to avoid self-occlusion */
 			gtk_window_set_type_hint (GTK_WINDOW (_corepointer_warning), GDK_WINDOW_TYPE_HINT_NORMAL);
 			gtk_window_set_position (GTK_WINDOW (_corepointer_warning), GTK_WIN_POS_CENTER);
 			gtk_widget_show_all (_corepointer_warning);
 		}
+		return TRUE;
 	}
+	return FALSE;
 }
 
 void
